@@ -81,12 +81,12 @@ struct obex_commands ftp = {
 	.chkput		= ftp_chkput,
 };
 
-static void os_reset_session(struct obex_session *os, gboolean aborted)
+static void os_reset_session(struct obex_session *os)
 {
 	if (os->fd > 0) {
 		close(os->fd);
 		os->fd = -1;
-		if (aborted && os->cmd == OBEX_CMD_PUT && os->current_folder) {
+		if (os->aborted && os->cmd == OBEX_CMD_PUT && os->current_folder) {
 			gchar *path;
 			path = g_build_filename(os->current_folder, os->name, NULL);
 			unlink(path);
@@ -105,22 +105,28 @@ static void os_reset_session(struct obex_session *os, gboolean aborted)
 		g_free(os->buf);
 		os->buf = NULL;
 	}
+	os->aborted = FALSE;
 	os->offset = 0;
 	os->size = OBJECT_SIZE_DELETE;
 }
 
+static void os_session_mark_aborted(struct obex_session *os)
+{
+	/* the session was alredy cancelled/aborted */
+	if (os->aborted)
+		return;
+
+	os->aborted = os->size == OBJECT_SIZE_UNKNOWN ? FALSE :
+							os->size != os->offset;
+}
+
 static void obex_session_free(struct obex_session *os)
 {
-	if (os->name)
-		g_free(os->name);
-	if (os->type)
-		g_free(os->type);
+	os_reset_session(os);
+
 	if (os->current_folder)
 		g_free(os->current_folder);
-	if (os->buf)
-		g_free(os->buf);
-	if (os->fd > 0)
-		close(os->fd);
+
 	g_free(os);
 }
 
@@ -232,7 +238,6 @@ static void cmd_connect(struct obex_session *os,
 			OBEX_FL_FIT_ONE_PACKET);
 
 	OBEX_ObjectSetRsp(obj, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
-
 }
 
 static gboolean chk_cid(obex_t *obex, obex_object_t *obj, guint32 cid)
@@ -359,27 +364,26 @@ static void cmd_setpath(struct obex_session *os,
 	}
 
 	while (OBEX_ObjectGetNextHeader(obex, obj, &hi, &hd, &hlen)) {
-		if (hi == OBEX_HDR_NAME) {
-			if (os->name) {
-				debug("Ignoring multiple name headers");
-				break;
-			}
+		if (hi != OBEX_HDR_NAME)
+			continue;
 
-			/*
-			 * This is because OBEX_UnicodeToChar() accesses
-			 * the string even if its size is zero
-			 */
-			if (hlen == 0) {
-				os->name = g_strdup("");
-				break;
-			}
-
-			os->name = g_convert((const gchar *) hd.bs, hlen,
-					"UTF8", "UTF16BE", NULL, NULL, NULL);
-
-			debug("Set path name: %s", os->name);
+		if (os->name) {
+			debug("Ignoring multiple name headers");
 			break;
 		}
+
+		/* This is because OBEX_UnicodeToChar() accesses the string
+		   even if its size is zero */
+		if (hlen == 0) {
+			os->name = g_strdup("");
+			break;
+		}
+
+		os->name = g_convert((const gchar *) hd.bs, hlen,
+				"UTF8", "UTF16BE", NULL, NULL, NULL);
+
+		debug("Set path name: %s", os->name);
+		break;
 	}
 
 	os->cmds->setpath(obex, obj);
@@ -424,7 +428,7 @@ static gint obex_write_stream(struct obex_session *os,
 		os->name ? os->name : "", os->type ? os->type : "",
 		os->tx_mtu, os->fd);
 
-	if (os->cancelled)
+	if (os->aborted)
 		return -EPERM;
 
 	if (os->fd < 0) {
@@ -511,9 +515,13 @@ static gint obex_read_stream(struct obex_session *os, obex_t *obex,
 	gint32 len = 0;
 	const guint8 *buffer;
 
-	if (os->cancelled)
+	if (os->aborted)
 		return -EPERM;
 
+	/* workaround: client didn't send the object lenght */
+	if (os->size == OBJECT_SIZE_DELETE)
+		os->size = OBJECT_SIZE_UNKNOWN;
+
 	size = OBEX_ObjectReadStream(obex, obj, &buffer);
 	if (size < 0) {
 		error("Error on OBEX stream");
@@ -666,8 +674,10 @@ static gboolean check_put(obex_t *obex, obex_object_t *obj)
 		return FALSE;
 	}
 
-	if (os->size == OBJECT_SIZE_DELETE)
+	if (os->size == OBJECT_SIZE_DELETE || os->size == OBJECT_SIZE_UNKNOWN) {
+		debug("Got a PUT without a Length");
 		goto done;
+	}
 
 	if (fstatvfs(os->fd, &buf) < 0) {
 		int err = errno;
@@ -724,7 +734,10 @@ static void obex_event(obex_t *obex, obex_object_t *obj, gint mode,
 			emit_transfer_progress(os->cid, os->size, os->offset);
 		break;
 	case OBEX_EV_ABORT:
-		os_reset_session(os, TRUE);
+		os->aborted = TRUE;
+		if (os->target == NULL)
+			emit_transfer_completed(os->cid, FALSE);
+		os_reset_session(os);
 		OBEX_ObjectSetRsp(obj, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
 		break;
 	case OBEX_EV_REQDONE:
@@ -734,10 +747,10 @@ static void obex_event(obex_t *obex, obex_object_t *obj, gint mode,
 			break;
 		case OBEX_CMD_PUT:
 		case OBEX_CMD_GET:
+			os_session_mark_aborted(os);
 			if (os->target == NULL)
-				emit_transfer_completed(os->cid,
-							os->offset == os->size);
-			os_reset_session(os, FALSE);
+				emit_transfer_completed(os->cid, !os->aborted);
+			os_reset_session(os);
 			break;
 		default:
 			break;
@@ -828,8 +841,7 @@ static void obex_handle_destroy(gpointer user_data)
 	if (os->target == NULL) {
 		/* Got an error during a transfer. */
 		if (os->fd >= 0)
-			emit_transfer_completed(os->cid,
-					os->offset == os->size);
+			emit_transfer_completed(os->cid, os->offset == os->size);
 
 		unregister_transfer(os->cid);
 	} else {
@@ -912,6 +924,7 @@ gint obex_session_start(gint fd, struct server *server)
 	g_io_add_watch_full(io, G_PRIORITY_DEFAULT,
 			G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
 			obex_handle_input, obex, obex_handle_destroy);
+	g_io_channel_set_close_on_unref(io, TRUE);
 	g_io_channel_unref(io);
 
 	return 0;
